/* AsmBuilder_x8632nasm.cpp */

#include "AsmBuilder_x8632nasm.hpp"

void AsmBuilder_x8632nasm::raw8(uint8_t val) {
	curSection->data += ".db " + std::to_string(val) + " \n";
}

void AsmBuilder_x8632nasm::raw16(uint16_t val) {
	curSection->data += ".dw " + std::to_string(val) + " \n";
}

void AsmBuilder_x8632nasm::raw32(uint32_t val) {
	curSection->data += ".dd " + std::to_string(val) + " \n";
}

void AsmBuilder_x8632nasm::rawData(const char* str, size_t len) {
	for (size_t i = 0; i < len; ++i) {
		raw8(str[i]);
	}
}

void AsmBuilder_x8632nasm::rawData(std::string str) {
	for (size_t i = 0; i < str.size(); ++i) {
		raw8(str[i]);
	}
}

/* */

std::string getNameForRegister(Register r) {
	switch (r) {
		case REG_A:  return "eax";
		case REG_C:  return "ecx";
		case REG_D:  return "edx";
		case REG_B:  return "ebx";
		case REG_SP: return "esp";
		case REG_BP: return "ebp";
		case REG_SI: return "esi";
		case REG_DI: return "edi";
	}
	return "INVALID";
}

void AsmBuilder_x8632nasm::pushReg(Register r) {
	curSection->data += "push " + getNameForRegister(r) + " \n";
}

void AsmBuilder_x8632nasm::popReg(Register r) {
	curSection->data += "pop " + getNameForRegister(r) + " \n";
}

void AsmBuilder_x8632nasm::pushRegOffset(Register r, int32_t offset) {
	curSection->data += "push dword[ " + getNameForRegister(r) + " + " + std::to_string(offset) + " ] \n";
}

void AsmBuilder_x8632nasm::movRegImm32(Register dest, uint32_t val) {
	curSection->data += "mov " + getNameForRegister(dest) + " , " + std::to_string(val) + " \n";
}

void AsmBuilder_x8632nasm::movRegReg(Register dest, Register src) {
	curSection->data += "mov " + getNameForRegister(dest) + " , " + getNameForRegister(src) + " \n";
}

void AsmBuilder_x8632nasm::movRegRegOffset(Register dest, Register srcptr, int32_t srcoffset) {
	curSection->data += "mov " + getNameForRegister(dest) + " , dword[ " + getNameForRegister(srcptr) + " + " + std::to_string(srcoffset) + " ] \n";
}

void AsmBuilder_x8632nasm::movRegOffsetReg(Register destptr, int32_t destoffset, Register src) {
	curSection->data += "mov dword[ " + getNameForRegister(destptr) + " + " + std::to_string(destoffset) + " ], " + getNameForRegister(src) + " \n";
}

void AsmBuilder_x8632nasm::addRegReg(Register accumulator, Register src) {
	curSection->data += "add " + getNameForRegister(accumulator) + " , " + std::to_string(src) + " \n";
}

void AsmBuilder_x8632nasm::addRegImm32(Register accumulator, uint32_t val) {
	curSection->data += "add " + getNameForRegister(accumulator) + " , " + std::to_string(val) + " \n";
}

void AsmBuilder_x8632nasm::clearRegister(Register r) {
	curSection->data += "xor " + getNameForRegister(r) + " , " + getNameForRegister(r) + " \n";
}

/* */

void AsmBuilder_x8632nasm::exportGlobalHere(std::string name) {
	curSection->data += "global " + name + "\n";
}

void AsmBuilder_x8632nasm::label(std::string name) {
	curSection->data += name + ": \n";
}

void AsmBuilder_x8632nasm::jumpUnconditional(std::string label) {
	curSection->data += "jmp " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfLesserOrEqual(std::string label) {
	curSection->data += "jle " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfLesser(std::string label) {
	curSection->data += "jl " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfEqual(std::string label) {
	curSection->data += "je " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfNotEqual(std::string label) {
	curSection->data += "jne " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfGreater(std::string label) {
	curSection->data += "jg " + label + " \n";
}

void AsmBuilder_x8632nasm::jumpIfGreaterOrEqual(std::string label) {
	curSection->data += "jge " + label + " \n";
}

void AsmBuilder_x8632nasm::noop() {
	curSection->data += "nop \n";
}

/* */

void AsmBuilder_x8632nasm::callByLabel(std::string label) {
	curSection->data += "call " + label + " \n";
}

void AsmBuilder_x8632nasm::callFromRegister(Register r) {
	curSection->data += "call " + getNameForRegister(r) + " \n";
}

void AsmBuilder_x8632nasm::leave() {
	curSection->data += "leave \n";
}

void AsmBuilder_x8632nasm::returnFunc() {
	curSection->data += "retn \n";
}

void AsmBuilder_x8632nasm::returnFunc(uint16_t stacksize) {
	curSection->data += "retn " + std::to_string(stacksize) + " \n";
}

/* */

bool AsmBuilder_x8632nasm::renderObject(std::string& ret) {
	ret.clear();
	ret += "BITS 32 \n";

	for (auto i = sections.begin(); i != sections.end(); ++i) {
		ret += "section " + i->first + " \n";
		ret += i->second.data;
	}

	return true;
}
